//---------------------------------------------------------------------------------------
//  EDBrowserCell.m created by erik on Tue Apr 02 2002
//  @(#)$Id: EDImageBrowserCell.m,v 1.2 2002/09/02 20:06:27 erik Exp $
//
//  Copyright (c) 2002 by Erik Doernenburg. All rights reserved.
//
//  Permission to use, copy, modify and distribute this software and its documentation
//  is hereby granted, provided that both the copyright notice and this permission
//  notice appear in all copies of the software, derivative works or modified versions,
//  and any portions thereof, and that both notices appear in supporting documentation,
//  and that credit is given to Mulle Kybernetik in all documents and publicity
//  pertaining to direct or indirect use of this code or its derivatives.
//
//  THIS IS EXPERIMENTAL SOFTWARE AND IT IS KNOWN TO HAVE BUGS, SOME OF WHICH MAY HAVE
//  SERIOUS CONSEQUENCES. THE COPYRIGHT HOLDER ALLOWS FREE USE OF THIS SOFTWARE IN ITS
//  "AS IS" CONDITION. THE COPYRIGHT HOLDER DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY
//  DAMAGES WHATSOEVER RESULTING DIRECTLY OR INDIRECTLY FROM THE USE OF THIS SOFTWARE
//  OR OF ANY DERIVATIVE WORK.
//---------------------------------------------------------------------------------------


#import "EDImageBrowserCell.h"

#define MARGIN 2.0


//---------------------------------------------------------------------------------------
    @implementation MKEDImageBrowserCell
//---------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------
//	DEALLOC
//---------------------------------------------------------------------------------------

- (void)dealloc
{
    [leftImage release];
    [rightImage release];
    [super dealloc];
}


//---------------------------------------------------------------------------------------
//	ACCESSOR METHODS
//---------------------------------------------------------------------------------------

- (void)setLeftImage:(NSImage *)anImage
{
    [anImage retain];
    [leftImage release];
    leftImage = anImage;
}

- (NSImage *)leftImage
{
    return leftImage;
}


- (void)setRightImage:(NSImage *)anImage
{
    [anImage retain];
    [rightImage release];
    rightImage = anImage;
}

- (NSImage *)rightImage
{
    return rightImage;
}


//---------------------------------------------------------------------------------------
//	ACCESSOR METHODS
//---------------------------------------------------------------------------------------

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
    NSImage			*li, *ri, *bi;
    unsigned int  	loff, roff;
    NSRect    		textRect;
    NSPoint			liOrigin, riOrigin, biOrigin;

    loff = roff = MARGIN;
    bi = nil;

    if((li = [self leftImage]) != nil)
        {
        loff += [li size].width + MARGIN;
        liOrigin.x = MARGIN;
        liOrigin.y = NSMinY(cellFrame) + NSHeight(cellFrame) - (NSHeight(cellFrame) - [li size].height) / 2.0;
        }
    if((ri = [self rightImage]) != nil)
        {
        roff += [ri size].width + MARGIN;
        riOrigin.x = NSMaxX(cellFrame) - MARGIN - [ri size].width;
        riOrigin.y = NSMinY(cellFrame) + NSHeight(cellFrame) - (NSHeight(cellFrame) - [ri size].height) / 2.0;
        }
    if([self isLeaf] == NO)
        {
        bi = [self isHighlighted] ? [[self class] highlightedBranchImage] : [[self class] branchImage];
        roff += [bi size].width + MARGIN;
        riOrigin.x -= [bi size].width + MARGIN;
        biOrigin.x = NSMaxX(cellFrame) - MARGIN - [bi size].width;
        biOrigin.y = NSMinY(cellFrame) + NSHeight(cellFrame) - (NSHeight(cellFrame) - [bi size].height) / 2.0;
        }

    // now draw background...
    if([self isHighlighted] || ([self state] == NSOnState))
        {
        [[self highlightColorInView:controlView] set];
        NSRectFill(cellFrame);
        }

    // ...text...
    textRect = cellFrame;
    textRect.origin.x += loff;
    textRect.size.width -= loff + roff;
    if(textRect.size.width > 0)
        [[self attributedStringValue] drawInRect:textRect];

    // ...and Images
    [li compositeToPoint:liOrigin operation:NSCompositeSourceOver];
    [ri compositeToPoint:riOrigin operation:NSCompositeSourceOver];
    [bi compositeToPoint:biOrigin operation:NSCompositeSourceOver];
}

//---------------------------------------------------------------------------------------
    @end
//---------------------------------------------------------------------------------------
