//---------------------------------------------------------------------------------------
//  MKConnectInspector.m created by erik on Tue Apr 02 2002
//  @(#)$Id: MKConnectInspector.m,v 1.7 2002/12/16 23:26:16 erik Exp $
//
//  Copyright (c) 2002 by Mulle Kybernetik. All rights reserved.
//
//  Permission to use, copy, modify and distribute this software and its documentation
//  is hereby granted, provided that both the copyright notice and this permission
//  notice appear in all copies of the software, derivative works or modified versions,
//  and any portions thereof, and that both notices appear in supporting documentation,
//  and that credit is given to Mulle Kybernetik in all documents and publicity
//  pertaining to direct or indirect use of this code or its derivatives.
//
//  THIS IS EXPERIMENTAL SOFTWARE AND IT IS KNOWN TO HAVE BUGS, SOME OF WHICH MAY HAVE
//  SERIOUS CONSEQUENCES. THE COPYRIGHT HOLDER ALLOWS FREE USE OF THIS SOFTWARE IN ITS
//  "AS IS" CONDITION. THE COPYRIGHT HOLDER DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY
//  DAMAGES WHATSOEVER RESULTING DIRECTLY OR INDIRECTLY FROM THE USE OF THIS SOFTWARE
//  OR OF ANY DERIVATIVE WORK.
//---------------------------------------------------------------------------------------

#import <AppKit/AppKit.h>
#import "IBPrivate.h"
#import "MKConnectionBrowserCell.h"
#import "MKPathNode.h"
#import "MKConnectInspector.h"
#import "MKNibControlConnectionType.h"
#import "MKNibOutletConnectionType.h"

@interface MKConnectInspector(PrivateAPI)
- (void)_getChildrenForNode:(MKPathNode *)parent;
- (void)_swapButton:(NSButton *)button toSelector:(SEL)selector;
@end


#define BEEPRETURN do { NSBeep(); return; } while (0)


//---------------------------------------------------------------------------------------
    @implementation MKConnectInspector
//---------------------------------------------------------------------------------------

static NSMutableArray *connectionTypeClasses = nil;


//---------------------------------------------------------------------------------------
//  CONNECTION TYPE CLASS LIST MANAGEMENT
//---------------------------------------------------------------------------------------

+ (void)initialize
{
    if(connectionTypeClasses != nil)
        return;
    connectionTypeClasses = [[NSMutableArray alloc] init];
    //[connectionTypeClasses addObject:NSClassFromString(@"MKEOConnectionType")];
    [connectionTypeClasses addObject:[MKNibControlConnectionType class]];
    [connectionTypeClasses addObject:[MKNibOutletConnectionType class]];
}


+ (void)registerConnectionTypeClass:(Class)aClass
{
    if([connectionTypeClasses containsObject:aClass])
        return;
    [connectionTypeClasses insertObject:aClass atIndex:0];
}


//---------------------------------------------------------------------------------------
//  INIT & DEALLOC
//---------------------------------------------------------------------------------------

- (id)init
{
    [super init];

    connectionTypes = [[NSMutableArray alloc] init];
    connectionTypeForSocket = [[NSMutableArray alloc] init];

    [NSBundle loadNibNamed:@"MKConnectInspector" owner:self];

    return self;
}


- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [warningImage release];
    [destinationPathRootNode release];
    [destinationPathOrigin release];
    [connectionTypes release];
    [connectionTypeForSocket release];
    [super dealloc];
}


//---------------------------------------------------------------------------------------
//	UI INITIALISATION
//---------------------------------------------------------------------------------------

- (void)awakeFromNib
{
    [socketBrowser setCellClass:[MKConnectionBrowserCell class]];
    [socketBrowser setFont:[destinationOutlineView font]];
    [socketBrowser setMaxVisibleColumns:1];
    [socketBrowser setDoubleAction:@selector(dclickSocket:)];
    [destinationOutlineView setIntercellSpacing:NSMakeSize(1.0, 1.0)];
    [destinationOutlineView setIndentationPerLevel:10];
    [destinationOutlineView setDoubleAction:@selector(dclickSocket:)];
    warningImage = [[warningIcon image] retain];
    [warningIcon setImage:nil];
    [warningText setStringValue:@""];
}


- (BOOL)isResizable
{
    return YES;
}


//---------------------------------------------------------------------------------------
//	STATE
//---------------------------------------------------------------------------------------

- (NSArray *)connectors
{
    return [[self inspectedDocument] connectorsForSource:inspectedObject];
}


- (NSArray *)activeConnectionTypes
{
    NSMutableArray		*activeConnectionTypes;
    NSEnumerator		*typeEnum;
    MKConnectionType	*type, *selectedType;

    activeConnectionTypes = [NSMutableArray array];
    selectedType = [[typePopUp selectedItem] representedObject];
    typeEnum = [connectionTypes objectEnumerator];
    while((type = [typeEnum nextObject]) != nil)
        {
        if(((selectedType != nil) && [type isEqual:selectedType]) || [type isKindOfClass:[MKNibControlConnectionType class]] || [type isKindOfClass:[MKNibOutletConnectionType class]])
            [activeConnectionTypes addObject:type];
        }
    return activeConnectionTypes;
}


- (NSArray *)socketLabels
{
    NSMutableArray	*socketLabels;
    NSEnumerator	*cellEnum;
    NSBrowserCell	*cell;

    socketLabels = [NSMutableArray array];
    cellEnum = [[[socketBrowser matrixInColumn:0] cells] objectEnumerator];
    while((cell = [cellEnum nextObject]) != nil)
        [socketLabels addObject:[cell stringValue]];

    return socketLabels;
}


- (NSString *)selectedSocketLabel
{
    return [[socketBrowser selectedCellInColumn:0] stringValue];
}


- (MKConnectionType *)connectionTypeForSelectedSocket
{
    int socketIndex;

    if((socketIndex = [socketBrowser selectedRowInColumn:0]) < 0)
        return nil;
    if(socketIndex >= [connectionTypeForSocket count]) // can happen during init
        return nil;
    return [connectionTypeForSocket objectAtIndex:socketIndex];
}


- (NSString *)selectedDestinationPath
{
    int	rowIdx;
    
    if((rowIdx = [destinationOutlineView selectedRow]) < 0)
        return nil;
    return [[destinationOutlineView itemAtRow:rowIdx] path];
}


- (id <MKIBConnectors>)selectedConnector
{
    int	selIdx;

    if((selIdx = [connectorTableView selectedRow]) < 0)
        return nil;
    return [[self connectors] objectAtIndex:selIdx];
}


//---------------------------------------------------------------------------------------
//	SELECT AN OBJECT
//---------------------------------------------------------------------------------------

- (void)revert:(id)sender
{    
    NSLog(@"%s", __PRETTY_FUNCTION__);

    if([self object] != inspectedObject)
        [self selectObject:sender];
    
    if([NSApp connectDestination] != destinationObject)
        [self selectConnectDestination:sender];

    [super revert:sender];
}


- (void)selectObject:(id)sender
{
    NSEnumerator	*typeClassEnum;
    Class			typeClass;

    // Remember inspected object
    [inspectedObject autorelease];
    inspectedObject = [[self object] retain];

    // Collect all connection types for this object
    [connectionTypes removeAllObjects];
    typeClassEnum = [connectionTypeClasses objectEnumerator];
    while((typeClass = [typeClassEnum nextObject]) != nil)
        [connectionTypes addObjectsFromArray:[typeClass connectionTypesForObject:inspectedObject]];
    [connectionTypes makeObjectsPerformSelector:@selector(setDocument:) withObject:[self inspectedDocument]];

    NSLog(@"%s %@ (%@)", __PRETTY_FUNCTION__, NSStringFromClass([inspectedObject class]), [(id <MKIBDocuments>)[self inspectedDocument] classForObject:inspectedObject]);

    [self redisplayTypePopUpKeepSelection:YES];
    [self redisplaySocketBrowserKeepSelection:NO];
    [self redisplayDestinationOutlineViewKeepSelection:NO];
    [self redisplayConnectorTableViewKeepSelection:NO];
    [self redisplayButtonsAndWarning];
}


- (void)selectConnectDestination:(id)sender
{
    // Remember destination object
    [destinationObject autorelease];
    destinationObject = [[NSApp connectDestination] retain];

    NSLog(@"%s %@ (%@)", __PRETTY_FUNCTION__, destinationObject, (destinationObject != nil) ? [(id <MKIBDocuments>)[self inspectedDocument] classForObject:[NSApp connectDestination]] : @"nil");

    if(destinationObject != nil)
        [self suggestSocket];
    
    [self redisplayDestinationOutlineViewKeepSelection:NO];
    [self redisplayButtonsAndWarning];
}


//---------------------------------------------------------------------------------------
//	TYPE POPUP
//---------------------------------------------------------------------------------------

- (void)redisplayTypePopUpKeepSelection:(BOOL)selFlag
{
    NSEnumerator		*typeEnum, *connectorEnum;
    MKConnectionType	*type, *previouslySelectedType;
    NSString			*requiredTypeName;
    id <MKIBConnectors>	connector;
    int					idx;

    // Remember previous type; if not the Outlets one
    previouslySelectedType = [[[[typePopUp selectedItem] representedObject] retain] autorelease];
    if([previouslySelectedType isKindOfClass:[MKNibOutletConnectionType class]])
        previouslySelectedType = nil;

    // Build PopUp from current connection types
    [typePopUp removeAllItems];
    typeEnum = [connectionTypes objectEnumerator];
    while((type = [typeEnum nextObject]) != nil)
        {
        // The control connection type returns nil here because it "hides" under the
        // outlet connection type. This is, of course, a special case...
        if([type name] == nil)
            continue;
        [typePopUp addItemWithTitle:[type name]];
        [[typePopUp lastItem] setRepresentedObject:type];
        }

    // See whether a specific type is required for the current connector list
    requiredTypeName = nil;
    connectorEnum = [[self connectors] objectEnumerator];
    while((connector = [connectorEnum nextObject]) != nil)
        {
        requiredTypeName = [connector connectionTypeName];
        if((requiredTypeName != nil) && ([requiredTypeName isEqualToString:@"Outlets"] == NO))
            break;
        }

    // Select an appropriate connection type
    if(requiredTypeName != nil)
        {
        // Ensure that some "funky" types (obsolete or private associations for examples) 
        // are added if required as the types might not want to list them by default. 
        if((idx = [typePopUp indexOfItemWithTitle:requiredTypeName]) == -1)
            {
            NSLog(@"** adding hidden connection type %@", requiredTypeName);
            [typePopUp addItemWithTitle:requiredTypeName];
            idx = [typePopUp indexOfItemWithTitle:requiredTypeName];
            }
        [typePopUp selectItemAtIndex:idx];
        }
    else if((selFlag) && (previouslySelectedType != nil) &&
        ((idx = [typePopUp indexOfItemWithTitle:[previouslySelectedType name]]) != -1))
        {
        [typePopUp selectItemAtIndex:idx];
        }
    else if([[typePopUp itemArray] count] > 0)
        {
        [typePopUp selectItemAtIndex:0];
        }
}


//---------------------------------------------------------------------------------------

- (IBAction)selectConnectionType:(id)sender
{
    [self redisplaySocketBrowserKeepSelection:NO];
    [self redisplayConnectorTableViewKeepSelection:YES];
    [self suggestSocket];
    [self redisplayDestinationOutlineViewKeepSelection:NO];
    [self redisplayButtonsAndWarning];
}


//---------------------------------------------------------------------------------------
//	BROWSER
//---------------------------------------------------------------------------------------

- (void)redisplaySocketBrowserKeepSelection:(BOOL)selFlag
{
    int socketIdx;

    socketIdx = [socketBrowser selectedRowInColumn:0];
    [socketBrowser loadColumnZero];
    if((selFlag) && (socketIdx >= 0))
        [socketBrowser selectRow:socketIdx inColumn:0];
}


- (void)browser:(NSBrowser *)sender createRowsForColumn:(int)column inMatrix:(NSMatrix *)matrix
{
    NSArray					*outletsTaken;
    MKConnectionType		*type;
    NSMutableDictionary		*connectorForLabel;
    NSEnumerator			*typeEnum, *connectorEnum, *labelEnum;
    NSString				*label;
    MKConnectionBrowserCell	*cell;
    id <MKIBConnectors>		connector;
    int						rowIdx;

    if(column > 0)
        return;

    // Collect all existing connectors and store them by their label
    connectorForLabel = [NSMutableDictionary dictionary];
    connectorEnum = [[self connectors] objectEnumerator];
    while((connector = [connectorEnum nextObject]) != nil)
        {
#warning * do something if the connector class does not respond to connectionTypeClass
        label = [connector sourceLabelForDocument:(id <MKIBDocuments>)[self inspectedDocument]];
        NSAssert([connectorForLabel objectForKey:label] == nil, @"Duplicate connector label");
        [connectorForLabel setObject:connector forKey:label];
        }

    // Now collect all labels from all content types and setup the browser cells
    outletsTaken = [[[typePopUp selectedItem] representedObject] socketLabelsTakenForObject:inspectedObject];
    rowIdx = 0;
    [connectionTypeForSocket removeAllObjects];
    typeEnum = [[self activeConnectionTypes] objectEnumerator];
    while((type = [typeEnum nextObject]) != nil)
        {
        labelEnum = [[type socketLabelsForObject:inspectedObject] objectEnumerator];
        while((label = [labelEnum nextObject]) != nil)
            {
            [matrix addRow];
            [connectionTypeForSocket addObject:type];
            cell = [matrix cellAtRow:rowIdx++ column:0];
            [cell setStringValue:label];
            [cell setRepresentedObject:[connectorForLabel objectForKey:label]];
            if((outletsTaken != nil) && ([outletsTaken indexOfObject:label] != NSNotFound))
                [cell setEnabled:NO];
            else
                [connectorForLabel removeObjectForKey:label];
            [type setupSocketBrowserCell:cell];
            }
        }

    // Finally, remove all incompatible connectors
    connectorEnum = [connectorForLabel objectEnumerator];
    while((connector = [connectorEnum nextObject]) != nil)
        [[self inspectedDocument] removeConnector:connector];
}


- (void)browserDidScroll:(NSBrowser *)sender
{
    [socketBrowser scrollColumnToVisible:0];
}


//---------------------------------------------------------------------------------------

- (void)suggestSocket
{
    NSArray				*preferredLabels, *socketLabels;
    MKConnectionType	*type;
    NSEnumerator		*typeEnum, *preferredLabelEnum;
    NSString			*label;
    NSCell				*socketCell;
    int					suggestedSocketIndex, socketIndex;

    socketLabels = [self socketLabels];
    suggestedSocketIndex = NSNotFound;
    typeEnum = [[self activeConnectionTypes] objectEnumerator];
    while((suggestedSocketIndex == NSNotFound) && ((type = [typeEnum nextObject]) != nil))
        {
        preferredLabels = [type preferredSocketLabelsForObject:inspectedObject destination:destinationObject];
        preferredLabelEnum = [preferredLabels objectEnumerator];
        while((suggestedSocketIndex == NSNotFound) && ((label = [preferredLabelEnum nextObject]) != nil))
            {
            socketIndex = [socketLabels indexOfObject:label];
            NSAssert(socketIndex != NSNotFound, @"Preferred lables not a subset of labels.");
            socketCell = [[[socketBrowser matrixInColumn:0] cells] objectAtIndex:socketIndex];
            if(([socketCell representedObject] == nil) && ([socketCell isEnabled]))
                suggestedSocketIndex = socketIndex;
            }
        }
    if(suggestedSocketIndex != NSNotFound)
        [socketBrowser selectRow:suggestedSocketIndex inColumn:0];
}


- (void)selectSocketForConnectorInSocketBrowser:(id <IBConnectors>)connector
{
    NSArray	*cells;
    int		i, n;

    cells = [[socketBrowser matrixInColumn:0] cells];
    for(i = 0, n = [cells count]; i < n; i++)
        {
        if((id <IBConnectors>)[[cells objectAtIndex:i] representedObject] == connector)
            break;
        }
    if(i < n)
        [socketBrowser selectRow:i inColumn:0];
}


//---------------------------------------------------------------------------------------

- (IBAction)selectSocket:(id)sender
{
    id <MKIBConnectors> selectedConnector;
    int					connectorIdx;

    if((selectedConnector = [[socketBrowser selectedCellInColumn:0] representedObject]) != nil)
        {
        connectorIdx = [[self connectors] indexOfObject:selectedConnector];
        NSAssert(connectorIdx != NSNotFound, @"Connector in browser but not in table view?!");
        [NSApp displayConnectionBetween:[selectedConnector source] and:[selectedConnector destination]];
        [destinationObject autorelease];
        destinationObject = [[NSApp connectDestination] retain];
        [connectorTableView selectRow:connectorIdx byExtendingSelection:NO];
        }
    else
        {
        [connectorTableView deselectAll:self];
        }

    [self redisplayDestinationOutlineViewKeepSelection:YES];
    if(selectedConnector != nil)
        [self showAndSelectPathInDestinationOutlineView:[selectedConnector destinationPath]];
    [self redisplayButtonsAndWarning];
}


- (IBAction)dclickSocket:(id)sender
{
    [connectButton performClick:self];
}


//---------------------------------------------------------------------------------------
//	DESTINATION OUTLINE VIEW
//---------------------------------------------------------------------------------------

- (void)redisplayDestinationOutlineViewKeepSelection:(BOOL)selFlag
{
    if(selFlag == NO)
        {
        [destinationOutlineView deselectAll:self];
        [destinationPathField setStringValue:@""];
        }
    [destinationOutlineView reloadData];
}


- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
    NSArray	*origin;
    
    if((destinationObject == nil) || ([[socketBrowser selectedCellInColumn:0] isLeaf]))
        return 0;

    if(item == nil)
        {
        origin = [NSArray arrayWithObjects:destinationObject, [self connectionTypeForSelectedSocket], nil];
        if((destinationPathOrigin == nil) || ([destinationPathOrigin isEqual:origin] == NO))
            {
            [destinationPathRootNode release];
            destinationPathRootNode = [[MKPathNode alloc] init];
            [destinationPathOrigin release];
            destinationPathOrigin = [origin retain];
            }
        item = destinationPathRootNode;
        }
    if([item children] == nil)
       [self _getChildrenForNode:item];
    
    return [[item children] count];
}


- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item
{
    if(item == nil)
        item = destinationPathRootNode;
    return [[item children] objectAtIndex:index];
}


- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
    if([item children] == nil)
        [self _getChildrenForNode:item];
    return [[item children] count] > 0;
}


- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
    return [item label];
}


- (void)outlineView:(NSOutlineView *)outlineView willDisplayOutlineCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
}


- (BOOL)outlineView:(NSOutlineView *)outlineView shouldCollapseItem:(id)item
{
    return YES;
}


- (void)_getChildrenForNode:(MKPathNode *)parent
{
    MKConnectionType	*type;
    NSMutableArray		*children;
    NSEnumerator		*labelEnum;
    NSString			*label;
    MKPathNode			*node;

    children = [NSMutableArray array];
    type = [self connectionTypeForSelectedSocket];
    labelEnum = [[type destinationLabelsForObject:destinationObject path:[parent path]] objectEnumerator];
    while((label = [labelEnum nextObject]) != nil)
        {
        node = [[[MKPathNode alloc] init] autorelease];
        [node setLabel:label];
        [node setParent:parent];
        [children addObject:node];
        }
    [parent setChildren:children];
}


//---------------------------------------------------------------------------------------

- (void)showAndSelectPathInDestinationOutlineView:(NSString *)path
{
    NSEnumerator	*pathEnum;
    NSString		*nextLabel, *label;
    MKPathNode		*item;
    int				i, level;

    [destinationPathField setStringValue:(path != nil) ? path : @""];

    if(path == nil)
        return;
    
    level = i = 0;
    pathEnum = [[path componentsSeparatedByString:@"."] objectEnumerator];
    label = [pathEnum nextObject];
    while(label != nil)
        {
        for(; i< [destinationOutlineView numberOfRows]; i++)
            {
            item = [destinationOutlineView itemAtRow:i];
            if(([[item label] isEqualToString:label]) && ([destinationOutlineView levelForItem:item] == level))
                break;
            }
        if(i == [destinationOutlineView numberOfRows])
            break;
        nextLabel = [pathEnum nextObject];
        if(nextLabel != nil)
            [destinationOutlineView expandItem:[destinationOutlineView itemAtRow:i]];
        label = nextLabel;
        level += 1;
        }

    // If we got to the end, select the last item
    if(label == nil)
        [destinationOutlineView selectRow:i byExtendingSelection:NO];
}


//---------------------------------------------------------------------------------------

- (IBAction)selectDestination:(id)sender
{
    NSString *path;
    
    NSLog(@"%s", __PRETTY_FUNCTION__);
    path = [self selectedDestinationPath];
    [destinationPathField setStringValue:(path != nil) ? path : @""];
    [self redisplayButtonsAndWarning];
}


- (IBAction)dclickDestination:(id)sender
{
    [connectButton performClick:self];
}


- (IBAction)setDestinationPath:(id)sender
{
#warning * implement me
}

//---------------------------------------------------------------------------------------
//	CONNECTOR TABLE VIEW
//---------------------------------------------------------------------------------------

- (void)redisplayConnectorTableViewKeepSelection:(BOOL)selFlag
{
    if(selFlag == NO)
        [connectorTableView deselectAll:self];
    [connectorTableView reloadData];
}


- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [[self connectors] count];
}


- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
    id <MKIBConnectors> connector;

    connector = [[self connectors] objectAtIndex:rowIndex];
    if([[aTableColumn identifier] isEqualToString:@"class"])
        return NSStringFromClass([connector class]);
    if([[aTableColumn identifier] isEqualToString:@"source"])
        return [connector sourceLabelForDocument:(id <MKIBDocuments>)[self inspectedDocument]];
    return [connector destinationLabelForDocument:(id <MKIBDocuments>)[self inspectedDocument]];
}


//---------------------------------------------------------------------------------------

- (void)selectConnectorInConnectorTableView:(id <IBConnectors>)connector
{
    [connectorTableView selectRow:[[self connectors] indexOfObject:connector] byExtendingSelection:NO];
}


//---------------------------------------------------------------------------------------

- (IBAction)selectConnector:(id)sender
{
    id <MKIBConnectors>	connector;

    if((connector = [self selectedConnector]) == nil)
        BEEPRETURN;

    [NSApp displayConnectionBetween:[connector source] and:[connector destination]];
    [destinationObject autorelease];
    destinationObject = [[NSApp connectDestination] retain];
    [self selectSocketForConnectorInSocketBrowser:connector];
    [self redisplayDestinationOutlineViewKeepSelection:NO];
    [self showAndSelectPathInDestinationOutlineView:[connector destinationPath]];
    [self redisplayButtonsAndWarning];
}


//---------------------------------------------------------------------------------------
//	BUTTONS AND WARNING TEXT
//---------------------------------------------------------------------------------------

- (void)redisplayButtonsAndWarning
{
    NSString	*message;
    NSCell 		*cell;

    [warningIcon setImage:nil];
    [warningText setStringValue:@""];

    cell = [socketBrowser selectedCellInColumn:0];
    if((cell != nil) && ([cell representedObject] != nil)) // have connector, can disconnect
        {
        [connectButton setEnabled:YES];
        [self _swapButton:connectButton toSelector:@selector(disconnect:)];
        }
    else if(cell != nil) // have selection, no connector, might be able to connect
        {
        [self _swapButton:connectButton toSelector:@selector(connect:)];
        message = nil; // better safe than sorry...
        if([[self connectionTypeForSelectedSocket] canConnectSource:inspectedObject socketLabel:[cell stringValue] destination:destinationObject path:[self selectedDestinationPath] message:&message])
            [connectButton setEnabled:YES];
        else
            [connectButton setEnabled:NO];
        if((message != nil) && ([message isEqualToString:@""] == NO))
            {
            [warningIcon setImage:warningImage];
            [warningText setStringValue:message];
            }
        }
    else
        {
        [connectButton setEnabled:NO];
        [self _swapButton:connectButton toSelector:@selector(connect:)];
        }
}


- (void)_swapButton:(NSButton *)button toSelector:(SEL)selector
{
    NSString	*temp;
    
    if([button action] == selector)
        return;
    temp = [[[button title] retain] autorelease];
    [button setTitle:[button alternateTitle]];
    [button setAlternateTitle:temp];
    [button setAction:selector];
}


//---------------------------------------------------------------------------------------

- (IBAction)connect:(id)sender
{
    MKConnectionType	*type;

    if((type = [self connectionTypeForSelectedSocket]) == nil)
        BEEPRETURN; // button should have been disabled!

    if([type addConnectorForSource:[NSApp connectSource] socketLabel:[self selectedSocketLabel] destination:destinationObject path:[self selectedDestinationPath]] == NO)
        BEEPRETURN;
    [[self inspectedDocument] touch];

    [self redisplaySocketBrowserKeepSelection:YES];
    [self redisplayDestinationOutlineViewKeepSelection:YES];
    [self redisplayConnectorTableViewKeepSelection:NO];
    [self selectConnectorInConnectorTableView:[[socketBrowser selectedCellInColumn:0] representedObject]];
    [self redisplayButtonsAndWarning];
}


- (IBAction)disconnect:(id)sender
{
    id <MKIBConnectors>	connector;

    if((connector = [self selectedConnector]) == nil)
        BEEPRETURN; // no connector selected, button should have been disabled!

    [[self inspectedDocument] removeConnector:connector];
    [[self inspectedDocument] touch];

    [self redisplaySocketBrowserKeepSelection:YES];
    [self redisplayDestinationOutlineViewKeepSelection:YES];
    [self redisplayConnectorTableViewKeepSelection:NO];
    [self redisplayButtonsAndWarning];
}


//---------------------------------------------------------------------------------------
    @end
//---------------------------------------------------------------------------------------



//---------------------------------------------------------------------------------------
    @implementation NSObject(MKConnectInspector)
//---------------------------------------------------------------------------------------

- (NSString *)connectInspectorClassName;
{
    //return @"IBConnectInspector";
    return @"MKConnectInspector";
}

//---------------------------------------------------------------------------------------
    @end
//---------------------------------------------------------------------------------------
 


