//---------------------------------------------------------------------------------------
//  MKConnectionBrowserCell.m created by erik on Fri Apr 12 2002
//  @(#)$Id: MKConnectionBrowserCell.m,v 1.2 2002/09/02 20:06:27 erik Exp $
//
//  Copyright (c) 2002 by Mulle Kybernetik. All rights reserved.
//
//  Permission to use, copy, modify and distribute this software and its documentation
//  is hereby granted, provided that both the copyright notice and this permission
//  notice appear in all copies of the software, derivative works or modified versions,
//  and any portions thereof, and that both notices appear in supporting documentation,
//  and that credit is given to Mulle Kybernetik in all documents and publicity
//  pertaining to direct or indirect use of this code or its derivatives.
//
//  THIS IS EXPERIMENTAL SOFTWARE AND IT IS KNOWN TO HAVE BUGS, SOME OF WHICH MAY HAVE
//  SERIOUS CONSEQUENCES. THE COPYRIGHT HOLDER ALLOWS FREE USE OF THIS SOFTWARE IN ITS
//  "AS IS" CONDITION. THE COPYRIGHT HOLDER DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY
//  DAMAGES WHATSOEVER RESULTING DIRECTLY OR INDIRECTLY FROM THE USE OF THIS SOFTWARE
//  OR OF ANY DERIVATIVE WORK.
//---------------------------------------------------------------------------------------

#import <AppKit/AppKit.h>
#import "MKConnectInspector.h"
#import "MKConnectionBrowserCell.h"

enum {
    MKConnectedImage,
    MKHighlightedConnectedImage,
    MKTriangleImage,
    MKDefaultConnectorImage,
    MKAttributeConnectorImage,
    MKToOneRelationshipConnectorImage,
    MKToManyRelationshipConnectorImage,
    MK_CICOUNT
};


//---------------------------------------------------------------------------------------
    @implementation MKConnectionBrowserCell
//---------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------
//	IMAGE CACHE
//---------------------------------------------------------------------------------------

#define getLocalImage(constant) getImage(constant, #constant, YES)


static NSImage *getImage(int constant, const char *name, BOOL findLocally)
{
    static NSImage *images[MK_CICOUNT];

    if(images[constant] == nil)
        {
        NSString	*nameString;
        NSBundle	*bundle;

        nameString = [NSString stringWithCString:name];
        bundle = [NSBundle bundleForClass:[MKConnectInspector class]];
        if(findLocally)
            images[constant] = [[NSImage alloc] initByReferencingFile:[bundle pathForImageResource:nameString]];
        else
            images[constant] = [[NSImage imageNamed:nameString] retain];
        NSCAssert1(images[constant] != nil, @"Can't find image \"%@\"", nameString);
        }
    return images[constant];
}


//---------------------------------------------------------------------------------------
//	WELLKNOWN IMAGES
//---------------------------------------------------------------------------------------

+ (NSImage *)connectedImage
{
    return getImage(MKConnectedImage, "knob_nib", NO);
}


+ (NSImage *)highlightedConnectedImage
{
    return getImage(MKHighlightedConnectedImage, "trackKnob_nib", NO);
}


+ (NSImage *)triangleImage
{
    return getLocalImage(MKTriangleImage);
}


+ (NSImage *)defaultConnectorImage
{
    return getLocalImage(MKDefaultConnectorImage);
}


+ (NSImage *)attributeConnectorImage
{
    return getLocalImage(MKAttributeConnectorImage);
}


+ (NSImage *)toOneRelationshipConnectorImage
{
    return getLocalImage(MKToOneRelationshipConnectorImage);
}


+ (NSImage *)toManyRelationshipConnectorImage
{
    return getLocalImage(MKToManyRelationshipConnectorImage);
}


//---------------------------------------------------------------------------------------
//	ACCESSORS
//---------------------------------------------------------------------------------------

- (void)setConnected:(BOOL)flag
{
    isConnected = flag;
}


- (BOOL)isConnected
{
    return isConnected;
}


- (NSImage *)rightImage
{
    if(isConnected == NO)
        return [super rightImage];
    return getImage(MKHighlightedConnectedImage, "trackKnob_nib", NO);
//    return ([self state] == NSOnState) ? getImage(MKHighlightedConnectedImage, "trackKnob_nib", NO) : getImage(MKConnectedImage, "knob_nib", NO);
}


- (NSSize)cellSize
{
    NSSize	mySize;

    mySize = [super cellSize];
    mySize.height += 1;

    return mySize;
}


//---------------------------------------------------------------------------------------
    @end
//---------------------------------------------------------------------------------------
