//---------------------------------------------------------------------------------------
//  MKConnectionType.m created by erik on Fri Apr 12 2002
//  @(#)$Id: MKConnectionType.m,v 1.7 2002/12/16 23:26:16 erik Exp $
//
//  Copyright (c) 2002 by Mulle Kybernetik. All rights reserved.
//
//  Permission to use, copy, modify and distribute this software and its documentation
//  is hereby granted, provided that both the copyright notice and this permission
//  notice appear in all copies of the software, derivative works or modified versions,
//  and any portions thereof, and that both notices appear in supporting documentation,
//  and that credit is given to Mulle Kybernetik in all documents and publicity
//  pertaining to direct or indirect use of this code or its derivatives.
//
//  THIS IS EXPERIMENTAL SOFTWARE AND IT IS KNOWN TO HAVE BUGS, SOME OF WHICH MAY HAVE
//  SERIOUS CONSEQUENCES. THE COPYRIGHT HOLDER ALLOWS FREE USE OF THIS SOFTWARE IN ITS
//  "AS IS" CONDITION. THE COPYRIGHT HOLDER DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY
//  DAMAGES WHATSOEVER RESULTING DIRECTLY OR INDIRECTLY FROM THE USE OF THIS SOFTWARE
//  OR OF ANY DERIVATIVE WORK.
//---------------------------------------------------------------------------------------

#import <AppKit/AppKit.h>
#import "MKConnectionType.h"


//=======================================================================================
    @implementation MKConnectionType
//=======================================================================================

static NSArray *emptyArray = nil;

+ (void)initialize
{
    if(emptyArray == nil)
        emptyArray = [[NSArray alloc] init];
}


//---------------------------------------------------------------------------------------
//	CLASS ATTRIBUTES
//---------------------------------------------------------------------------------------

+ (NSString *)sourceLabelForConnector:(id <IBConnectors>)connector
{
    return nil;
}


+ (NSString *)destinationLabelForConnector:(id <IBConnectors>)connector
{
    return nil;
}


//---------------------------------------------------------------------------------------
//	FACTORY
//---------------------------------------------------------------------------------------

+ (NSArray *)connectionTypesForObject:(id)anObject
{
    return [NSArray arrayWithObject:[[[[self class] alloc] init] autorelease]];
}


//---------------------------------------------------------------------------------------
//	ISEQUAL
//---------------------------------------------------------------------------------------

- (BOOL)isEqual:(id)other
{
    return [[self name] isEqualToString:[other name]];
}


//---------------------------------------------------------------------------------------
//	ACCESSORS
//---------------------------------------------------------------------------------------

- (void)setDocument:(id <MKIBDocuments>)aDocument
{
    document = aDocument;
}


- (id <MKIBDocuments>)document
{
    return document;
}


//---------------------------------------------------------------------------------------
//	DEFAULT IMPLEMENTATIONS
//---------------------------------------------------------------------------------------

- (NSString *)name
{
    return NSStringFromClass([self class]);
}


- (NSArray *)socketLabelsForObject:(id)anObject
{
    return emptyArray;
}


- (NSArray *)socketLabelsTakenForObject:(id)anObject
{
    return emptyArray;
}


- (void)setupSocketBrowserCell:(MKConnectionBrowserCell *)cell
{
}


- (NSArray *)preferredSocketLabelsForObject:(id)anObject destination:(id)otherObject
{
    return emptyArray;
}


- (NSArray *)destinationLabelsForObject:(id)anObject path:(NSString *)path
{
    return nil;
}


- (BOOL)canConnectSource:(id)sourceObject socketLabel:(NSString *)label destination:(id)destObject path:(NSString *)path message:(NSString **)messagePtr
{
#warning * string should be localized
    *messagePtr = @"this connection should not be made";
    return NO;
}


- (NSString *)connectionTypeRequiredForConnector:(id <IBConnectors>)connector
{
    return nil;
}


- (id <IBConnectors>)connectorForSource:(id)source socketLabel:(NSString *)label destination:(id)destObject path:(NSString *)path
{
    return nil;
}


- (BOOL)addConnectorForSource:(id)source socketLabel:(NSString *)label destination:(id)destObject path:(NSString *)path
{
    id <IBConnectors> connector;

    if((connector = [self connectorForSource:source socketLabel:label destination:destObject path:path]) != nil)
        [document addConnector:connector];
    
    return (connector != nil);
}


+ (BOOL)objectIsFirstResponder:(id)anObject
{
    return [anObject isKindOfClass:NSClassFromString(@"IBFirstResponder")];
}


//=======================================================================================
    @end
//=======================================================================================



//=======================================================================================
    @implementation NSObject(MKConnectionInfo)
//=======================================================================================

- (NSArray *)outletLabelsForDocument:(id <MKIBDocuments>)document
{
    return [document outletsOfClass:[document classForObject:self]];
}


- (NSArray *)actionLabelsForDocument:(id <MKIBDocuments>)document
{
    NSMutableArray	*labels;
    NSEnumerator	*actionEnum;
    NSString		*action;

    labels = [NSMutableArray array];
    actionEnum = [[document actionsOfClass:[document classForObject:self]] objectEnumerator];
    while((action = [actionEnum nextObject]) != nil)
        [labels addObject:[action stringByAppendingString:@":"]];
    
    return labels;
}


- (BOOL)followsTargetActionParadigmForDocument:(id <MKIBDocuments>)document
{
    return [document objectFollowsTargetActionParadigm:self];
}


//=======================================================================================
    @end
//=======================================================================================
