//---------------------------------------------------------------------------------------
//  MKEOConnectionType.m created by erik on Fri Apr 12 2002
//  @(#)$Id: MKEOConnectionType.m,v 1.4 2002/09/02 20:20:29 erik Exp $
//
//  Copyright (c) 2002 by Mulle Kybernetik. All rights reserved.
//
//  Permission to use, copy, modify and distribute this software and its documentation
//  is hereby granted, provided that both the copyright notice and this permission
//  notice appear in all copies of the software, derivative works or modified versions,
//  and any portions thereof, and that both notices appear in supporting documentation,
//  and that credit is given to Mulle Kybernetik in all documents and publicity
//  pertaining to direct or indirect use of this code or its derivatives.
//
//  THIS IS EXPERIMENTAL SOFTWARE AND IT IS KNOWN TO HAVE BUGS, SOME OF WHICH MAY HAVE
//  SERIOUS CONSEQUENCES. THE COPYRIGHT HOLDER ALLOWS FREE USE OF THIS SOFTWARE IN ITS
//  "AS IS" CONDITION. THE COPYRIGHT HOLDER DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY
//  DAMAGES WHATSOEVER RESULTING DIRECTLY OR INDIRECTLY FROM THE USE OF THIS SOFTWARE
//  OR OF ANY DERIVATIVE WORK.
//---------------------------------------------------------------------------------------

#import <EOInterface/EOInterface.h>
#import <EOInterface/EOAspectConnector.h>
#ifdef WO5
#import <EOPlaceholder/EOPlaceholder.h>
#endif
#import <EDCommon/EDCommon.h>
#import "MKConnectionBrowserCell.h"
#import "MKEOConnectionType.h"


//---------------------------------------------------------------------------------------
    @implementation MKEOConnectionType
//---------------------------------------------------------------------------------------

+ (NSArray *)connectionTypesForObject:(id)anObject
{
    NSMutableArray		*typeList;
    MKEOConnectionType	*type;
    NSEnumerator		*associationClassEnum;
    Class				cls;

    typeList = [NSMutableArray array];
    associationClassEnum = [[EOAssociation associationClassesForObject:anObject] objectEnumerator];
    while((cls = [associationClassEnum nextObject]) != nil)
        {
        type = [[[[self class] alloc] init] autorelease];
        type->associationClass = cls;
        [typeList addObject:type];
        }
    return [typeList sortedArrayByComparingAttribute:@"name"];
}


//---------------------------------------------------------------------------------------

- (NSString *)name
{
    return [associationClass displayName];
}


//---------------------------------------------------------------------------------------

- (NSArray *)socketLabelsForObject:(id)anObject
{
    return [associationClass aspects];
}


- (NSArray *)preferredSocketLabelsForObject:(id)anObject destination:(id)otherObject
{
    NSMutableArray	*preferredLabelList;

    // primary aspect can be *nil* which is handled
    preferredLabelList = [NSMutableArray arrayWithObjects:[associationClass primaryAspect], nil];
    // we might add the primary aspect twice but that doesn't do any harm
    [preferredLabelList addObjectsFromArray:[self socketLabelsForObject:anObject]];
    return preferredLabelList;
}


- (NSArray *)socketLabelsTakenForObject:(id)anObject
{
    return [associationClass objectKeysTaken];
}


- (void)setupSocketBrowserCell:(MKConnectionBrowserCell *)cell
{
    NSArray			*signatureList;
    NSString		*label, *signature;
    NSImage			*signatureSymbol;
    unsigned int	idx;

    label = [cell stringValue];
    if((idx = [[associationClass aspects] indexOfObject:label]) != NSNotFound)
        {
        signatureList = [associationClass aspectSignatures];
        signature = ([signatureList count] > idx) ? [signatureList objectAtIndex:idx] : @"A1M";
        if([signature hasPrefix:@"A"])
            signatureSymbol = [MKConnectionBrowserCell attributeConnectorImage];
        else if([signature hasPrefix:@"1"])
            signatureSymbol = [MKConnectionBrowserCell toOneRelationshipConnectorImage];
        else if([signature hasPrefix:@"M"])
            signatureSymbol = [MKConnectionBrowserCell toManyRelationshipConnectorImage];
        else
            signatureSymbol = [MKConnectionBrowserCell defaultConnectorImage];
        [cell setLeftImage:signatureSymbol];

        if([signature isEqualToString:@""])
            {
            [cell setLeaf:YES];
            [cell setConnected:([cell representedObject] != nil)];
            }
        }
}


//---------------------------------------------------------------------------------------

- (NSArray *)destinationLabelsForObject:(id)anObject path:(NSString *)path
{
    NSMutableArray		*allKeys;
    EOClassDescription	*classDescription;
    NSEnumerator		*pathEnum;
    NSString			*key;

    if([anObject isKindOfClass:[EODisplayGroup class]] == NO)
        return nil;
    
    classDescription = [[anObject dataSource] classDescriptionForObjects];
    pathEnum = [[path componentsSeparatedByString:@"."] objectEnumerator];
    while((key = [pathEnum nextObject]) != nil)
        classDescription = [classDescription classDescriptionForDestinationKey:key];

    allKeys = [NSMutableArray array];
    [allKeys addObjectsFromArray:[classDescription attributeKeys]];
    [allKeys addObjectsFromArray:[classDescription toOneRelationshipKeys]];
    [allKeys addObjectsFromArray:[classDescription toManyRelationshipKeys]];

    return allKeys;
}


//---------------------------------------------------------------------------------------

- (EOAssociation *)_associationForConnectors:(NSArray *)connectorList
{
    NSEnumerator 		*connectorEnum;
    id <MKIBConnectors>	connector;

    connectorEnum = [connectorList objectEnumerator];
    while((connector = [connectorEnum nextObject]) != nil)
        {
        if([[connector connectionTypeName] isEqualToString:[self name]])
            break;
        }
    return [(EOAspectConnector *)connector association]; 
}


- (BOOL)canConnectSource:(id)sourceObject socketLabel:(NSString *)label destination:(id)destObject path:(NSString *)path message:(NSString **)messagePtr
{
    EOAssociation 		*existingAssociation;
    EOClassDescription	*classDescription;
    NSArray				*signatureList;
    NSEnumerator		*pathEnum;
    NSString			*key, *lastKey, *signature;
    unsigned int		idx;
    
    *messagePtr = nil;

    // No destination, no problem (yet)
    if(destObject == nil)
        return NO;
    
    // get signature for aspect
    idx = [[associationClass aspects] indexOfObject:label];
    NSAssert1(idx != NSNotFound, @"nothing known about aspect %@", label);
    signatureList = [associationClass aspectSignatures];
    signature = ([signatureList count] > idx) ? [signatureList objectAtIndex:idx] : @"A1M";

    // are we connected to a display group?
    if([destObject isKindOfClass:[EODisplayGroup class]] == NO)
        {
        if([signature isEqualToString:@""])
            *messagePtr = [NSString stringWithFormat:@"%@ must be of type EODisplayGroup", label];
        return YES; // I have my doubts about EOActionAssciation; so let the user decide
        }

    // is this okay with the already connected aspects?
    existingAssociation = [self _associationForConnectors:existingConnectors];
    if((existingAssociation != nil) && ([existingAssociation canBindAspect:label displayGroup:destObject key:path] == NO))
        {
        *messagePtr = @"incompatible with other aspects";
        return NO;
        }

    // no path, only allow if a group is needed, no warning otherwise for the moment
    if(path == nil)
        {
        return ([signature isEqualToString:@""]);
        }

    // try and follow the key path, drop out on to-may relationship
    classDescription = [[destObject dataSource] classDescriptionForObjects];
    lastKey = nil;
    pathEnum = [[path componentsSeparatedByString:@"."] objectEnumerator];
    while((key = [pathEnum nextObject]) != nil)
        {
        if(lastKey != nil)
            classDescription = [classDescription classDescriptionForDestinationKey:lastKey];
        lastKey = key;
        if([[classDescription toManyRelationshipKeys] containsObject:key])
            break;
        }

    // found a to-many and not at end
    if((key != nil) && ([pathEnum nextObject] != nil))
        {
        // an exception would be "@count" ...
        *messagePtr = @"can only follow to-one releationships";
        return NO;
        }

    // check signature against key type
    if([[classDescription attributeKeys] containsObject:lastKey])
        {
        if([signature rangeOfString:@"A"].location != NSNotFound)
            return YES;
        }
    else if([[classDescription toOneRelationshipKeys] containsObject:lastKey])
        {
        if([signature rangeOfString:@"1"].location != NSNotFound)
            return YES;
        }
    else if([[classDescription toManyRelationshipKeys] containsObject:lastKey])
        {
        if([signature rangeOfString:@"M"].location != NSNotFound)
            return YES;
        }
    else
        {
        return NO;
        }

    // did not match, describe required type
    if([signature isEqualToString:@"A"])
        *messagePtr = [NSString stringWithFormat:@"%@ must be an attribute", label];
    else if(([signature isEqualToString:@"1M"]) || ([signature isEqualToString:@"M1"]))
        *messagePtr = [NSString stringWithFormat:@"%@ must be a releationship", label];
    else if([signature isEqualToString:@"1"])
        *messagePtr = [NSString stringWithFormat:@"%@ must be a to-one releationship", label];
    else if([signature isEqualToString:@"M"])
        *messagePtr = [NSString stringWithFormat:@"%@ must be a to-many releationship", label];
    else
        *messagePtr = [NSString stringWithFormat:@"%@ aspect has signature %@", label, signature];
    NSLog(@"%s %@ %@", __PRETTY_FUNCTION__, signature, *messagePtr);
    
    return NO;
}


- (BOOL)addConnectorForSource:(id)source socketLabel:(NSString *)label destination:(id)destObject path:(NSString *)path
{
    NSArray		  		*existingConnectors;
    EOAssociation 		*association;
    EOAspectConnector	*connector;

    existingConnectors = [document connectorsForSource:source];
    if((association = [self _associationForConnectors:existingConnectors]) == nil)
        association = [[[associationClass alloc] initWithObject:source] autorelease];
    NSAssert([association object] == source, @"different source objects?!");
    [association bindAspect:label displayGroup:destObject key:path];
    connector = [[[EOAspectConnector alloc] initWithAssociation:association aspectName:label] autorelease];
    if(connector == nil)
        return NO;
    [document addConnector:connector];
    return YES;
}


//=======================================================================================
    @end
//=======================================================================================



//=======================================================================================
    @implementation EOAspectConnector(MKConnectionType)
//=======================================================================================

- (NSString *)sourceLabelForDocument:(id <MKIBDocuments>)document
{
    return [self aspectName];
}


- (NSString *)destinationLabelForDocument:(id <MKIBDocuments>)document
{
    return [NSString stringWithFormat:@"%@ (%@)", [self destinationKey], [document nameForObject:[[self association] displayGroupForAspect:[self aspectName]]]];
}


- (NSString *)destinationPath
{
    return [self destinationKey];
}


- (NSString *)connectionTypeName
{
    return [[[self association] class] displayName];
}


//=======================================================================================
    @end
//=======================================================================================


#ifdef WO5
//=======================================================================================
    @implementation _EOAssociationAspectConnector(MKConnectionType)
//=======================================================================================

- (NSString *)sourceLabelForDocument:(id <IBDocuments>)document
{
    return [self aspect];
}


- (NSString *)destinationLabelForDocument:(id <IBDocuments>)document
{
    return [NSString stringWithFormat:@"%@ (%@)", [self key], [document nameForObject:[self destination]]];
}


- (NSString *)connectionTypeName
{
    return [[[self association] class] displayName];
}


//=======================================================================================
    @end
//=======================================================================================
#endif
